@@ -1,15 +1,15 @@
 #![deny(warnings)]
 #![deny(unused_lifetimes)]
-
 // No unsafe code should ever be used in derive macros.
 #![forbid(unsafe_code)]
 
 use proc_macro::TokenStream;
 use syn::{parse_macro_input, DeriveInput};
 
-mod encode;
-mod decode;
 mod common;
+mod decode;
+mod encode;
+mod enumeration;
 
 #[proc_macro_derive(EncodableMessage, attributes(otopr))]
 pub fn derive_encodable_message(ts: TokenStream) -> TokenStream {
@@ -24,3 +24,10 @@ pub fn derive_decodable_message(ts: TokenStream) -> TokenStream {
         .unwrap_or_else(|e| e.into_compile_error())
         .into()
 }
+
+#[proc_macro_derive(Enumeration)]
+pub fn derive_enumeration(ts: TokenStream) -> TokenStream {
+    enumeration::derive_enumeration(parse_macro_input!(ts as DeriveInput))
+        .unwrap_or_else(|e| e.into_compile_error())
+        .into()
+}
